#!/usr/bin/perl -w
##
# HW implementation for 'DebugNumber'.
#
# This is a simple hardware implementation of a handler which will write
# a number to the screen when a value is written to the address.
#
# @file
# @author Justin Fletcher
#

package DExecHW::DebugNumber;


##
# Construct a new hardware device object.
#
# @param[in]  $proto   This class object, or prototype to add to
# @param[in]  $object  The object to proxy for
# @param[in]  $offset  The offset this object is created for
#
# @return Assembler object
sub new
{
    my $proto = shift;
    my $object = shift;
    my $offset = shift || 0;
    my $class = ref($proto) || $proto;
    my $self = {
            'value' => 0,
        };

    bless $self, $class;
    
    return $self;
}


##
# Return the size of of the implementation.
#
# @param[in] $self   Object
#
# @return Size of the hardware implementation in words
sub window
{
    return 1;
}


##
# Read value
#
# @param[in] $self   Object
# @param[in] $offset Offset of register within window
# @param[in] $exec   DExec object opon which this object is operating
#
# @return value of this register
sub read
{
    my ($self, $offset, $exec) = @_;
    
    return $self->{'value'};
}


##
# Write value
#
# @param[in] $self   Object
# @param[in] $offset Offset of register within window
# @param[in] $exec   DExec object upon which this object is operating
# @param[in] $value  Value to write
#
# @return ignored
sub write
{
    my ($self, $offset, $exec, $value) = @_;
    
    print "$value\n";
    $self->{'value'} = $value;
}


##
# Get the symbol name for an offset
#
# @param[in] $self   Object
# @param[in] $offset Offset of register within window
#
# @return symbol name to use
# @retval undef for a default symbol
# @retval "" for no symbol
sub symbol
{
    my ($self, $offset) = @_;
    
    return "Print";
}


# Must return 1
1;
