#!/usr/bin/perl -w
##
# This script makes the README.html file which we place at the top level
# so that we can use it to document the source itself.
#
# We take the produced documentation index, and the CSS file, and we
# merge them into a single output file.
#

# Read in the content file
my $html = readfile("Docs/html/index.html");

# and the CSS
my $css = readfile("Docs/html/doxygen.css");

# Include the CSS in the HTML
$html =~ s/<\/head>/<style type='text\/css'>$css<\/style>\n<\/head>/;

# Remove the links to the CSS
$html =~ s/<link href=".*?" rel="stylesheet" type="text\/css" *\/>//g;

# Remove the tabs from the top
$html =~ s/<div id="navrow1" class="tabs">.*?<\/div>//s;

# Remove all links to things that aren't fully qualified objects
$html =~ s/<a [^>]*href="(?!http).*?"[^>]*>(.*?)<\/a>/$1/sg;

# Remove the footer image from the bottom (we retain the text)
$html =~ s/<img class="footer" src="doxygen.png" alt="doxygen"\/>/Doxygen/;

# Write out the object file
open(my $fh, ">", "README.html");
print $fh $html;
close($fh);

##
# Read a file in its entirity.
#
# @param[in] $file   File to read
#
# @return contents of the file, or dies if cannot read
sub readfile
{
    my ($file) = @_;
    open(my $fh, "<", $file) || die "Failed to read $file: $!";
    my $data;
    if (sysread($fh, $data, -s $file) != -s $file)
    {
        die "Failed to read all of $file: $!\n";
    }
    close($fh);
    
    return $data;
}
