#!/usr/bin/perl -w
##
# Test rules for the test-assemble directory: positive tests.
#
# These routines provide regression tests to ensure that the assemble
# routines generate the same results in repeated runs.
# We process the files in this directories:
#
#    *.expect.dump
#        The file *.dasm is assembled and then dumped.
#        The result is compared to this file.
#        If they do not match, the differences are printed and we terminate.
#        This tests that the assembler is generating the expected output
#        and that the dumped output is the same.
#
#    *.expect.dasm
#        The file *.dasm is assembled and then disassembled.
#        The result is compared to this file.
#        If they do not match, the differences are printed and we terminate.
#        This tests that the assembly and disassembly are consistent.
#        This file is /also/ assembled and compared to the *.expect.dump
#        file.


##
# Assemble the .dasm file and dump it; comparing to the expectation.
addTest('name' => 'Assemble and dump',
        'input'    => '(?<!expect)\.dasm$',
        'expect'   => '.expect.dump',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .dump -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );

##
# Assemble the .dasm file and disassemble it; compare to the expectation.
addTest('name' => 'Assemble and Disassemble',
        'input'  => '(?<!expect)\.dasm$',
        'expect' => '.expect.dasm',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .dasm -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


##
# Assemble the .dasm file, disassemble it, reassemble and dump it;
# Compare to the expectation.
addTest('name' => 'Assemble, Disassemble and Dump',
        'input'  => '(?<!expect)\.dasm$',
        'expect' => '.expect.dump',
        'generate' => sub {
                my ($directory, $input) = @_;

                # Run the assembly
                unlink "XXX.dasm";
                my $cmd1 = "./dasm.pl -o XXX.dasm -i $directory/$input";
                my $cmd2;
                my ($rc, $output) = capture_output($cmd1);

                if ($rc == 0)
                {
                    # Only continue if the assembly was successful
                    $cmd2 = "./dasm.pl -o .dump -i XXX.dasm";
                    my $moreoutput;
                    ($rc, $moreoutput) = capture_output($cmd2);
                    $output .= $moreoutput;
                    unlink "XXX.dasm";
                }

                return ($rc, $output, "$cmd1\n$cmd2");
            },

        'compare' => \&compare,
       );

# Must return true
1;
