#!/usr/bin/perl -w
##
# Test rules for the test-binary directory: positive tests of binary
# load/save
#
# This script provides regression tests to ensure that binary load
# routines produce the right results when loaded.
#
#    *.expect.dump
#        The file *.bin is loaded and then dumped.
#        The result is compared to this output.
#        If they do not match, the differences are printed and we terminate.
#        This tests that the binary load is the correct endianess
#
#    *.expect.bin
#        The file *.dump is loaded and then written to a binary output.
#        The result is compared to this output.
#        If they do not match, the locations are printed and we terminate.
#        This tests that the binary load is the correct endianess
#        This is then repeated for the *.expect.binle for little endian
#        format.
#        And again repeated for the *.expect.binbe for big endian
#        format.
#
#    *.dasm
#        The file is assembled and then written to a binary file. It is
#        then compared to the *.expect.bin file.
#        If they do not match the differences are printed and we 
#        terminate.


##
# Assemble the .dasm file and write to a binary; comparing to the expectation.
addTest('name' => 'Assemble and write binary',
        'input'    => '(?<!expect)\.dasm$',
        'expect'   => '.expect.bin',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .bin -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&binarycompare,
       );

##
# Load the .bin file and dump it; comparing to the expectation.
addTest('name' => 'Load binary and dump',
        'input'    => '(?<!expect)\.bin$',
        'expect'   => '.expect.dump',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .dump -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );

##
# Load a binary file and write out a little-endian binary
addTest('name' => 'Load binary and write little-endian',
        'input'    => '(?<!expect)\.bin$',
        'expect'   => '.expect.binle',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .binle -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


##
# Load a binary file and write out a big-endian binary
addTest('name' => 'Load binary and write big-endian',
        'input'    => '(?<!expect)\.bin$',
        'expect'   => '.expect.binbe',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .binbe -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


# Must return true
1;
