#!/usr/bin/perl -w
##
# Test rules for the test-comparison directory: miscellaneous tests.
#
# Run the comparison suite from:
#   http://0x10cwiki.com/wiki/Comparison_of_Developer_Tools
# through the compiler to check that we conform well.
#
# The initial scores for DASM were:
#
# Group                    : Pass : Fail : Total
# ------------------------ : ---- : ---- : ----
# Errors                   :   17 :   13 :   30
# Instructions             :    1 :    0 :    1
# Notch                    :    0 :    1 :    1
# Operands                 :    0 :    1 :    1
# Test Code                :    1 :    0 :    1
# _TOTAL_                  :   19 :   15 :   34
#
# Failures are caused by:
#  * The tests assume that the order of the offset and register in the
#    indexed operand format is reversible. The specification states that
#    the format is 'nextword + register', not the converse.
#    Action: Update assembler to allow the 'register + nextword' format.
#
#  * errors-15 uses 'DIV a,0' as a test. This is an INVALID test as the
#    specification explicitly states that this is acceptable and results
#    in both a and O being set to 0. This may be a reasonable operation,
#    and produces a word in memory of 0x8002 which may be used for some
#    magic code.
#    Action: None. Test is invalid, and has been removed.
#
#  * errors-12 allows 'jsr a,a', when it should not be accepted.
#    errors-11 also fails for the same reason.
#    Action: Fix existing code; had a missing '^' anchor when checking
#    the following line content. A few places in the instruction and symbol
#    processing had omitted anchors; these were updated to correct the
#    checks.
#
#  * errors-3 checks for too large a constant, which is being accepted.
#    Action: Update the checks in the operand processing for constant
#    values being outside the range. Allow -ve values for -0x10000 to -1,
#    though, because they are useful.
#
#  * Test code was incorrectly penalising the first 'error' output for
#    the output generated by the previous test.
#    Action: Fixed the test code to look at the right file.
#
# Improvements based on results:
#  * 'Parse error: Could not resolve all symbols to write out Address file'
#    is produced if there are undefined symbols.
#    Action: The report now includes the unresolved symbols.
#
# These tests are mostly a subset of those in the test-assemble and
# test-assemble-fail directories.
#


##
# Assemble the .dasm file and run it; comparing to the expectation.
addTest('name' => 'Assemble, checking output',
        'input'    => '(?<!errors-\d\d)(?<!.expect)\.dasm$',
        'expect'   => '.expect.dasm',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .dasm -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


##
# Assemble the .dasm file, checking for expected failures
addTest('name' => 'Assemble, expecting failures',
        'input'    => '(?<=errors-\d\d)\.dasm$',
        'expect'   => '.fail.output',
        'generate' => sub {
                my ($directory, $input) = @_;

                my $cmd = "./dasm.pl -o .dump -i $directory/$input";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&comparefail,
       );


# Must return true
1;
