#!/usr/bin/perl -w
##
# Test rules for the test-frontend directory: positive tests of execution
# of the front end.
#
# *.cmd command to execute.
# *.expect.output is what we expect to see.
#


##
# Run the command and check it works and matches expectations
addTest('name' => 'Frontend tool',
        'input'    => '.sh',
        'expect'   => '.expect.output',
        'no-expect-skip' => 1,  # Skip test if no expect file

        'generate' => sub {
                my ($directory, $input) = @_;
                
                my $content = readfile("$directory/$input");
                
                # Split up the command files into lines
                my @lines = grep { !/^\s*$/ && !/^\s*#/ } split /\n/, $content;

                my ($rc, $output) = (1, "");
                my $cmd = "";
                
                # Run every line and capture the output.
                for my $line (@lines)
                {
                    my $moreoutput;
                    $cmd .= "$line\n";
                    ($rc, $moreoutput) = capture_output($line);
                    $output .= $moreoutput;
                    
                    last if ($rc != 0);
                }

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


##
# Run the command and check it works and matches expectations
addTest('name' => 'Frontend tool failures',
        'input'    => '.sh',
        'expect'   => '.fail.output',
        'no-expect-skip' => 1,  # Skip test if no expect file

        'generate' => sub {
                my ($directory, $input) = @_;
                
                my $content = readfile("$directory/$input");
                
                # Split up the command files into lines
                my @lines = grep { !/^\s*$/ && !/^\s*#/ } split /\n/, $content;

                my ($rc, $output) = (1, "");
                my $cmd = "";
                
                # Run every line and capture the output.
                for my $line (@lines)
                {
                    my $moreoutput;
                    $cmd .= "$line\n";
                    ($rc, $moreoutput) = capture_output($line);
                    $output .= $moreoutput;
                    
                    last if ($rc != 0);
                }

                return ($rc, $output, $cmd);
            },

        'compare' => \&comparefail,
       );


# Must return true
1;
