#!/usr/bin/perl -w
##
# Test rules for the test-interactive directory: positive tests of execution
# and output.
#
#    *.expect.output
#        The file *.dasm is assembled and run.
#        The script is run, using the *.input file as standard input.
#        The output is written to a file.
#        The result is compared to this file.
#        If they do not match, the differences are printed and we terminate.
#        This tests that the code execution is as expected - at least in
#        terms of the terminal registers.
#
#    *.expect.screen
#        The file *.dasm is assembled and run.
#        The output is captured from within 'screen', as a 'hardcopy' of
#        the final screen.
#        If they do not match, the differences are printed and we terminate.
#        This tests that the code execution is as expected - at least in
#        terms of the terminal registers.
#


##
# Assemble the .dasm file and run it, with input redirected if present.
addTest('name' => 'Assemble and run interactively',
        'input'    => '(?<!expect)\.dasm$',
        'expect'   => '.expect.output',
        'no-expect-skip' => 1,
        'generate' => sub {
                my ($directory, $input) = @_;
                
                # If there's a '.input' file, we use it
                my $stdin = $input;
                $stdin =~ s/\.dasm/\.input/;
                if (-f "$directory/$stdin")
                {
                    $stdin = "< $directory/$stdin";
                }
                else
                {
                    $stdin = "";
                }

                my $cmd = "./dasm.pl -showregs -nocounts -notimes -run -i $directory/$input $stdin";
                my ($rc, $output) = capture_output($cmd);

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


##
# Assemble the .dasm file and run it, with input redirected if present.
addTest('name' => 'Assemble and run interactive screen',
        'input'    => '(?<!expect)\.dasm$',
        'expect'   => '.expect.screen',
        'no-expect-skip' => 1,
        'generate' => sub {
                my ($directory, $input) = @_;
                
                # If there's a '.input' file, we use it
                my $stdin = $input;
                $stdin =~ s/\.dasm/\.input/;
                if (-f "$directory/$stdin")
                {
                    $stdin = "$directory/$stdin";
                }
                else
                {
                    $stdin = "/dev/null";
                }
                
                
                my $cmd = "screen -c $directory/screenrc -S dcpu16-test -D -m ";
                $cmd .= "test-interactive/capture.sh $stdin XXX.screen 80 32 ";
                $cmd .= "./dasm.pl -nocount -notimes -run -i $directory/$input";

                my ($rc, $output) = capture_output($cmd);
                if (!-f "XXX.screen")
                {
                    # repeat the operation if the screen didn't appear -
                    # seems to fail now and then on screen '4.00.03jw4' which
                    # I use here.
                    ($rc, $output) = capture_output($cmd);
                }

                $output .= readfile("XXX.screen");
                unlink "XXX.screen";

                return ($rc, $output, $cmd);
            },

        'compare' => \&compare,
       );


# Must return true
1;
